<?php
/**
 * @package            Joomla
 * @subpackage         Event Booking
 * @author             Tuan Pham Ngoc
 * @copyright          Copyright (C) 2010 - 2018 Ossolution Team
 * @license            GNU/GPL, see LICENSE.php
 */

/**
 * This is a CRON script to delete expired session data which should be called from the command-line, not the
 * web. For example something like:
 * /usr/bin/php /path/to/site/cli/ebreminder.php
 */

// Initialize Joomla framework
const _JEXEC = 1;

// Load system defines
if (file_exists(dirname(__DIR__) . '/defines.php'))
{
	require_once dirname(__DIR__) . '/defines.php';
}

if (!defined('_JDEFINES'))
{
	define('JPATH_BASE', dirname(__DIR__));
	require_once JPATH_BASE . '/includes/defines.php';
}

// Get the framework.
require_once JPATH_LIBRARIES . '/import.legacy.php';

// Bootstrap the CMS libraries.
require_once JPATH_LIBRARIES . '/cms.php';

/**
 * Cron job to trash expired session data.
 *
 * @since  3.8.6
 */
class EBDepositReminder extends JApplicationCli
{
	/**
	 * Entry point for the script
	 *
	 * @return  void
	 *
	 * @since   3.8.6
	 */
	public function doExecute()
	{
		if (!file_exists(JPATH_ROOT . '/components/com_eventbooking/eventbooking.php'))
		{
			return;
		}

		// Require library + register autoloader
		require_once JPATH_ADMINISTRATOR . '/components/com_eventbooking/libraries/rad/bootstrap.php';

		$params = $this->loadEBDepositReminderPluginParams();

		$bccEmail                = $params->get('bcc_email', '');
		$numberDays              = (int) $params->get('number_days', 7);
		$numberEmailSendEachTime = (int) $params->get('number_registrants', 0);

		EventbookingHelperMail::sendDepositReminder($numberDays, $numberEmailSendEachTime, $bccEmail);
	}

	/**
	 * Load EB Reminder Plugin Params
	 *
	 * @return \Joomla\Registry\Registry
	 */
	private function loadEBDepositReminderPluginParams()
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select($db->quoteName('params'))
			->from('#__extensions')
			->where('`element` = "ebdepositreminder"')
			->where('`folder`="system"');
		$db->setQuery($query);

		return new \Joomla\Registry\Registry($db->loadResult());
	}
}

JApplicationCli::getInstance('EBDepositReminder')->execute();